@@ -10,6 +10,7 @@ from sql.functions import CharLength
 from stdnum import get_cc_module
 from stdnum.eu.vat import MEMBER_STATES as EU_MEMBER_STATES
 
+from trytond import backend
 from trytond.i18n import gettext
 from trytond.model import (
     DeactivableMixin, Index, ModelSQL, ModelView, MultiValueMixin, Unique,
@@ -141,28 +142,22 @@ class Party(
     def get_code_readonly(self, name):
         return True
 
-    @classmethod
-    def tax_identifier_types(cls):
-        return TAX_IDENTIFIER_TYPES
-
-    def get_tax_identifier(self, name):
-        types = self.tax_identifier_types()
+    def _get_identifier(self, name, types):
         for identifier in self.identifiers:
             if identifier.type in types:
                 return identifier.id
 
     @classmethod
-    def search_tax_identifier(cls, name, clause):
+    def _search_identifier(cls, name, clause, types):
         _, operator, value = clause
         nested = clause[0][len(name) + 1:]
-        types = cls.tax_identifier_types()
         domain = [
             ('identifiers', 'where', [
                     (nested or 'rec_name', operator, value),
                     ('type', 'in', types),
                     ]),
             ]
-        # Add party without tax identifier
+        # Add party without identifier
         if ((operator == '=' and value is None)
                 or (operator == 'in' and None in value)):
             domain = ['OR',
@@ -174,19 +169,32 @@ class Party(
                 ]
         return domain
 
+    @classmethod
+    def tax_identifier_types(cls):
+        return TAX_IDENTIFIER_TYPES
+
+    def get_tax_identifier(self, name):
+        types = self.tax_identifier_types()
+        return self._get_identifier(name, types)
+
+    @classmethod
+    def search_tax_identifier(cls, name, clause):
+        types = cls.tax_identifier_types()
+        return cls._search_identifier(name, clause, types)
+
     def get_full_name(self, name):
         return self.name
 
     def get_contact_mechanism(self, name):
-        for mechanism in self.contact_mechanisms:
-            if mechanism.type == name:
-                return mechanism.value
-        return ''
+        usage = Transaction().context.get('party_contact_mechanism_usage')
+        mechanism = self.contact_mechanism_get(name, usage)
+        return mechanism.value if mechanism else ''
 
     @classmethod
     def set_contact_mechanism(cls, parties, name, value):
         pool = Pool()
         ContactMechanism = pool.get('party.contact_mechanism')
+        usage = Transaction().context.get('party_contact_mechanism_usage')
         contact_mechanisms = []
         for party in parties:
             if getattr(party, name):
@@ -196,10 +204,22 @@ class Party(
                         party=party.rec_name,
                         field=type_string))
             if value:
-                contact_mechanisms.append(ContactMechanism(
+                contact_mechanism = None
+                if usage:
+                    for contact_mechanism in party.contact_mechanisms:
+                        if (contact_mechanism.type == name
+                                and contact_mechanism.value == value):
+                            break
+                    else:
+                        contact_mechanism = None
+                if not contact_mechanism:
+                    contact_mechanism = ContactMechanism(
                         party=party,
                         type=name,
-                        value=value))
+                        value=value)
+                if usage:
+                    setattr(contact_mechanism, usage, True)
+                contact_mechanisms.append(contact_mechanism)
         ContactMechanism.save(contact_mechanisms)
 
     @classmethod
@@ -382,7 +402,6 @@ class Party(
                 try:
                     result = eu_vat.check_vies(code)
                 except Exception:
-                    print('exception', code)
                     logger.debug(
                         f"Fail to check {identifier['code']}", exc_info=True)
                     continue
@@ -420,12 +439,17 @@ class PartyLang(ModelSQL, ValueMixin):
 class PartyCategory(ModelSQL):
     'Party - Category'
     __name__ = 'party.party-party.category'
-    _table = 'party_category_rel'
     party = fields.Many2One(
         'party.party', "Party", ondelete='CASCADE', required=True)
     category = fields.Many2One(
         'party.category', "Category", ondelete='CASCADE', required=True)
 
+    @classmethod
+    def __register__(cls, module):
+        # Migration from 7.0: rename to standard name
+        backend.TableHandler.table_rename('party_category_rel', cls._table)
+        super().__register__(module)
+
 
 IDENTIFIER_VAT = [
     'ad_nrt',
@@ -909,18 +933,22 @@ class CheckVIES(Wizard):
                     else:
                         parties_succeed.append(party.id)
                 except Exception as e:
-                    if hasattr(e, 'faultstring') \
-                            and hasattr(e.faultstring, 'find'):
-                        if e.faultstring.find('INVALID_INPUT'):
+                    for msg in e.args:
+                        if msg == 'INVALID_INPUT':
                             parties_failed.append(party.id)
-                            continue
-                        if e.faultstring.find('SERVICE_UNAVAILABLE') \
-                                or e.faultstring.find('MS_UNAVAILABLE') \
-                                or e.faultstring.find('TIMEOUT') \
-                                or e.faultstring.find('SERVER_BUSY'):
+                            break
+                        elif msg in {
+                                'SERVICE_UNAVAILABLE',
+                                'MS_UNAVAILABLE',
+                                'MS_MAX_CONCURRENT_REQ',
+                                'GLOBAL_MS_MAX_CONCURRENT_REQ',
+                                'TIMEOUT',
+                                'SERVER_BUSY',
+                                }:
                             raise VIESUnavailable(
                                 gettext('party.msg_vies_unavailable')) from e
-                    raise
+                    else:
+                        raise
         self.result.parties_succeed = parties_succeed
         self.result.parties_failed = parties_failed
         return 'result'
@@ -1071,7 +1099,8 @@ class Erase(Wizard):
     def transition_erase(self):
         pool = Pool()
         Party = pool.get('party.party')
-        cursor = Transaction().connection.cursor()
+        transaction = Transaction()
+        cursor = transaction.connection.cursor()
 
         resources = self.get_resources()
         parties = replacing = [self.ask.party]
@@ -1124,6 +1153,7 @@ class Erase(Wizard):
                                         Model.__name__ + ',%')
                                     & resource_field.sql_id(
                                         table.resource, Model).in_(query)))
+        transaction.counter += 1
         return 'end'
 
     def check_erase(self, party):
